#ifndef SHIM_EFI_TPCM_HEADER
#define SHIM_EFI_TPCM_HEADER 1

#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/dh.h>
#include <openssl/ocsp.h>
#include <openssl/pkcs12.h>
#include <openssl/rand.h>
#include <openssl/crypto.h>
#include <openssl/ssl.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/rsa.h>
#include <openssl/dso.h>

#include <Library/BaseCryptLib.h>

#define EFI_TPCM_GUID                                       \
    {                                                       \
        0xa37e200e, 0xda90, 0x473b,                         \
        {                                                   \
            0x8b, 0xb5, 0x1d, 0x7b, 0x11, 0xba, 0x32, 0x33  \
        }                                                   \
    }
#define SHIM_EFI_HASH2_SERVICE_BINDING_PROTOCOL_GUID        \
    {                                                       \
        0xda836f8d, 0x217f, 0x4ca0,                         \
        {                                                   \
            0x99, 0xc2, 0x1c, 0xa4, 0xe1, 0x60, 0x77, 0xea  \
        }                                                   \
    }
#define SHIM_EFI_HASH2_PROTOCOL_GUID                        \
    {                                                       \
        0x55b1d734, 0xc5e1, 0x49db,                         \
        {                                                   \
            0x96, 0x47, 0xb1, 0x6a, 0xfb, 0xe, 0x30, 0x5b   \
        }                                                   \
    }
#define SHIM_HASH_ALGORITHM_SM3_GUID                        \
    {                                                       \
        0x9DCD754B, 0x3479, 0x27AD,                         \
        {                                                   \
            0x56, 0x4C, 0x68, 0x7C, 0x68, 0xEC, 0xF9, 0xB9  \
        }                                                   \
    }

#define OEM_SIG_SIZE 3
#define FIRMWARE_VERSION_SIZE 32
#define FIRMWARE_HASH_CONYENT_SIZE 32
#define FIRMWARE_NAME_SIZE 32
#define SHIM_IPMI_TIMEOUT_MS 7000

#define IPMI_BMC_LUN 0x00
/* Net Function Definition */
#define IPMI_NETFN_OEM 0x30

#define IPMI_CMD_GET_MEASURE_PARM 0x92  // change a name

#define IPMI_SUB_CMD_MEASURE_REQ 0x57  // change a name
#define IPMI_SUB_CMD_CONTROL_REQ 0x58
#define IPMI_SUB_CMD_SWITCH_REQ 0x59

#define DEFAULT_HASH_SIZE 32
#define MEASURE_DATA_MEM_SIZE 0x100000

#define TPCM_TAG_REQ_COMMAND 0x000000C1
#define TPCM_ORD_ExternSimpleBootMeasure 0x00001053

typedef struct {
    UINT32 uiCmdTag;
    UINT32 uiCmdLength;
    UINT32 uiCmdCode;
    UINT32 uiPcr;
    UINT32 uiStage;
    UINT8 uaDigest[DEFAULT_HASH_SIZE];
    UINT32 uiObjLen;
    UINT8 uaObj[FIRMWARE_NAME_SIZE];
} extern_simple_bmeasure_req_st;

typedef struct {
    UINT8 OemSignature[OEM_SIG_SIZE];
    UINT8 SubCmd;
    UINT8 FirmwareType;
    UINT8 FirmwareDetailType;
    UINT8 FirmwareHashAlgorithmType;
    UINT8 FirmwareHashLen;
    extern_simple_bmeasure_req_st FirmwareHashContent;
    UINT8 FirmwareVerionLen;
    UINT8 FirmwareVerion[FIRMWARE_VERSION_SIZE];
} OEM_BMC_MEASURE_REQUSET;

typedef struct {
    UINT8 CompletionCode;
    UINT8 OemSignature[OEM_SIG_SIZE];
} OEM_BMC_MEASURE_RESPONSE;

typedef struct {
    UINT8 OemSignature[OEM_SIG_SIZE];
    UINT8 SubCmd;
    UINT8 FirmwareType;
    UINT8 FirmwareDetailType;
} OEM_BMC_GET_RESULT_REQUSET;

typedef struct {
    UINT8 OemSignature[OEM_SIG_SIZE];
    UINT8 ControlResult;
} OEM_BMC_GET_RESULT_RESPONSE;

typedef enum {
    IPMI_SYSTEM_INTERFACE_UNKNOWN,  // IPMI_SYSTEM_INTERFACE_TYPE->UNKNOWN
    IPMI_SYSTEM_INTERFACE_KCS,
    IPMI_SYSTEM_INTERFACE_SMIC,
    IPMI_SYSTEM_INTERFACE_BT,  // IPMI_SYSTEM_INTERFACE_TYPE->BT
    IPMI_SYSTEM_INTERFACE_SSIF,
    IPMI_SYSTEM_INTERFACE_MAX_TYPE  // IPMI_SYSTEM_INTERFACE_TYPE->MAX_TYPE
} shim_ipmi_system_interface_type;

typedef struct {
    UINT8 lun : 2;
    UINT8 net_fn : 6;
    UINT8 cmd;
} shim_ipmi_cmd_header;

typedef enum {
    IPMI_MEMORY,
    IPMI_IO,
    IPMI_MAX_INTERFACE_ADDRESS_TYPE
} shim_ipmi_interface_address_type;

typedef enum {
    IPMI_FW_SHIM,
    IPMI_FW_SHIM_GRUB,
    IPMI_FW_OS
} shim_ipmi_firmware_type;

typedef enum {
    IPMI_FW_DETAIL_GRUB,
    RESERVED1,
    RESERVED2
} shim_ipmi_firmware_detail_type;

typedef enum {
    SM3_HASH,
    HASH_RESERVED1,
    HASH_RESERVED2
} shim_ipmi_firmware_hash_type;

typedef enum {
    IPMI_MEASURE_UNKNOW,
    IPMI_MEASURE_SUCCESS,
    IPMI_MEASURE_FAIL
} shim_ipmi_measure_result_type;

typedef enum {
    IPMI_SWITCH_UNKNOW,
    IPMI_SWITCH_OPEN,
    IPMI_SWITCH_CLOSE
} shim_ipmi_get_switch_result_type;

typedef union {
    UINT8 Md5Hash[16];
    UINT8 Sha1Hash[20];
    UINT8 Sha224Hash[28];
    UINT8 Sha256Hash[32];
    UINT8 Sha384Hash[48];
    UINT8 Sha512Hash[64];
    UINT8 Sm3Hash[32];
} shim_efi_hash2_output;

struct shim_efi_hash2_protocol {
    EFI_STATUS (*get_hash_size)(struct shim_efi_hash2_protocol *this, EFI_GUID *hash_algorithm, UINT64 hash_size);

    EFI_STATUS (*hash)
    (struct shim_efi_hash2_protocol *this, EFI_GUID *hash_algorithm, UINT8 *message, UINT64 message_size,
        shim_efi_hash2_output *hash);

    EFI_STATUS (*hash_init)(struct shim_efi_hash2_protocol *this, EFI_GUID *hash_algorithm);

    EFI_STATUS (*hash_update)(struct shim_efi_hash2_protocol *this, UINT8 *message, UINT64 message_size);

    EFI_STATUS (*hash_final)(struct shim_efi_hash2_protocol *this, shim_efi_hash2_output *hash);
};

typedef struct shim_efi_hash2_protocol shim_efi_hash2_protocol_t;

struct shim_efi_ipmi_interface_protocol {
    EFI_STATUS (*excute_ipmi_cmd)
    (struct shim_efi_ipmi_interface_protocol *this, shim_ipmi_cmd_header request, void *send_data, UINT8 send_length,
        void *recv_data, UINT8 *recv_length, UINT16 *status_codes);
    shim_ipmi_system_interface_type (*get_ipmi_interface_type)(struct shim_efi_ipmi_interface_protocol *this);
    UINT16 (*get_ipmi_base_address)(struct shim_efi_ipmi_interface_protocol *this);
    shim_ipmi_interface_address_type (*get_ipmi_base_address_type)(struct shim_efi_ipmi_interface_protocol *this);
    UINT8 (*get_ipmi_version)(struct shim_efi_ipmi_interface_protocol *this);
};

typedef struct shim_efi_ipmi_interface_protocol shim_efi_ipmi_interface_protocol_t;

EFI_STATUS tpcm_measure_grub(void *context, unsigned char *buf, size_t size, EFI_HANDLE image_handle);

#endif
